/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <capdev_file.h>
#include <s_util.h>
#include <sys/mman.h>

PLUGIN_DECLARE_STD_IMPORTS;
proc_add_captype add_captype;

void *lfile_init(char *interface, char *filter)
{
	struct pcap_file_header *fh;
	struct stat st;
	p_cfile_priv p=my_malloc(sizeof(cfile_priv_t));

	if (p == NULL) return NULL;

	/* Open libpcap file */
	if ( (p->fd=open(interface, O_RDONLY)) < 0 ) {
		my_free(p);
		print_out(PRINT_ERROR "open(%s) failed" CRLF, interface);
		return NULL;
	}

	/* Stat it because we need the size to map */
	if ( fstat(p->fd, &st) != 0 ) {
		close(p->fd);
		my_free(p);
		print_out(PRINT_ERROR "fstat(%s) failed" CRLF, interface);
		return NULL;
	}

	/* Memory map that sucker */
	p->map_size=st.st_size;
	if ( (p->map=mmap(NULL, p->map_size, PROT_READ, MAP_SHARED, p->fd, 0))==MAP_FAILED ) {
		close(p->fd);
		my_free(p);
		print_out(PRINT_ERROR "mmap(%s) failed" CRLF, interface);
		return NULL;
	}

	
	fh=(struct pcap_file_header *)p->map;

#if 0
	print_out ( "LFILE: magic = %x" CRLF, fh->magic );
	print_out ( "LFILE: major = %i" CRLF, fh->version_major);
	print_out ( "LFILE: minor = %u" CRLF, fh->version_minor);
	print_out ( "LFILE: splen = %u" CRLF, fh->snaplen);
#endif
	
	if ( fh->magic == 0xa1b2cd34 && sizeof(struct pcap_pkthdr) != 24)  {
		print_raw(PRINT_WARNING "LBL libpcap, RedHat file. "
			"Engaging fixup." CRLF);
		p->fix = FIXUP_LBL;
	}else if ( fh->magic == 0xa1b2c3d4 && sizeof(struct pcap_pkthdr) == 24){
		print_raw(PRINT_WARNING "RedHat libpcap, LBL file. "
			"Engaging fixup." CRLF);
		p->fix = FIXUP_REDHAT;
	}
	
	p->snaplen=fh->snaplen;
	p->link=fh->linktype;
	p->cur=p->map;
	p->cur+=sizeof(struct pcap_file_header);
	
	return p;	
}

int lfile_end(void *priv)
{
	p_cfile_priv p=(p_cfile_priv)priv;
	if ( p == NULL ) return 0;
	munmap(p->map, p->map_size);
	close(p->fd);
	my_free(p);
	return 1;
}

struct packet *lfile_poll(void *priv)
{
	struct pcap_pkthdr *h;
	p_cfile_priv p=(p_cfile_priv)priv;

	if ( p == NULL ) return NULL;

	/* TODO: Better checking */
	if ( (unsigned long)p->cur > (unsigned long)p->map + p->map_size ) {
		return NULL;
	}
	
	h=(struct pcap_pkthdr *)p->cur;
	p->cur+=sizeof(struct pcap_pkthdr) + p->fix;

	p->pkt.time = h->ts.tv_sec;
	p->pkt.nano_time = h->ts.tv_usec;
	p->pkt.data=p->cur;
	p->cur+=h->caplen;
	
	p->pkt.caplen = h->caplen;
	p->pkt.len = h->len;
	p->pkt.layer[0].id=p->link;
	
	return &p->pkt;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_captype=in->add_captype;
	
	PLUGIN_NAME("capture/libpcap_file");
	PLUGIN_DESC("Capture packets from libpcap (tcpdump) files");
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	PLUGIN_VERS(1,0);
	
	if ( add_captype("libpcap_file", lfile_init, lfile_end, lfile_poll) ) {
		return PLUGIN_ERR_OK;
	}else{
		return PLUGIN_ERR_FAIL;
	}
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
