/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <match_icmp.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;

proc_matcher_add	add_matcher;

#define CHECK_ICMP(x) \
	(( TRANSPORT_LAYER(x).id == IPPROTO_ICMP ) ? 1 : 0)

int icmp_id_match(void *priv, p_packet p, int negate)
{
	ushort id;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_ICMP(p) ) return 0;

	id = ntohs(TRANSPORT_LAYER(p).h.icmp->un.echo.id);

	if ( (id >= x->min) && (id <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

int icmp_seq_match(void *priv, p_packet p, int negate)
{
	ushort seq;
	p_range_priv x=(p_range_priv)priv;
	
	if ( !CHECK_ICMP(p) ) return 0;

	seq = ntohs(TRANSPORT_LAYER(p).h.icmp->un.echo.seq);

	if ( (seq >= x->min) && (seq <= x->max) ) {
		MATCH_RET_POSITIVE;
	}else{
		MATCH_RET_NEGATIVE;
	}
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;

	add_matcher=in->add_matcher;

	PLUGIN_NAME("match/icmp");
	PLUGIN_DESC("Generic matches for ICMP packets");
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	PLUGIN_VERS(1,0)
	
	add_matcher("ICMP_ID",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		icmp_id_match,
		in->templates.t_cmp_range);

	add_matcher("ICMP_SEQ",
		in->templates.t_validate_range,
		in->templates.t_cleanup_ptr,
		icmp_seq_match,
		in->templates.t_cmp_range);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
