/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <ip.h>

PLUGIN_DECLARE_STD_IMPORTS;
proc_demux_add_handler add_demux;

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;

	PLUGIN_NAME("proto/tcpip");
	PLUGIN_DESC("TCP/IP networking protocols suite");
	PLUGIN_VERS(1,1);
	
	add_demux=in->add_demux;

	if ( !(add_demux(LTYPE_NET, demux_callback)) ) {
		return PLUGIN_ERR_FAIL;
	}

	if ( !(add_demux(LTYPE_TRANS, demux_callback2)) ) {
		return PLUGIN_ERR_FAIL;
	}
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}

#define NPROTO_IP	0x0800

/* IP itself */
int demux_callback(p_packet p, int layer)
{
	if ( p->layer[layer].id == NPROTO_IP ) {
		p->layer[layer].name = "IPv4";
		p->layer[layer+1].id = p->layer[layer].h.ip->protocol;
		p->layer[layer+1].h.raw = p->layer[layer].h.raw  + 
			p->layer[layer].h.ip->ihl*4;
		
		switch ( p->layer[layer].h.ip->protocol ) {
			case IPPROTO_GRE:
			case IPPROTO_IPIP:
			case IPPROTO_ENCAP:
			case 115: //IPPROTO_L2TP:
			case 131: //IPPROTO_PIPE:
				p->layer[layer+1].type = LTYPE_ENCAP;
				break;
			default:
				p->layer[layer+1].type = LTYPE_TRANS;
				break;
		}
		
		return DEMUX_OK;
	}
	
	return DEMUX_UNKNOWN;
}

/* All our adjuncts */
int demux_callback2(p_packet p, int layer)
{
	/* These transport layers are for IP only*/
	if ( p->layer[layer-1].id != NPROTO_IP ) return DEMUX_UNKNOWN;

	switch ( p->layer[layer].id ) {
		case IPPROTO_TCP:
			p->layer[layer].name = "TCP";
			p->sport = ntohs(p->layer[layer].h.tcp->sport);
			p->dport = ntohs(p->layer[layer].h.tcp->dport);
			p->layer[layer+1].h.raw = 
				p->layer[layer].h.raw + 
				p->layer[layer].h.tcp->doff;
			break;
		
		case IPPROTO_UDP:
			p->layer[layer].name = "UDP";
			p->sport = ntohs(p->layer[layer].h.udp->sport);
			p->dport = ntohs(p->layer[layer].h.udp->dport);
			p->layer[layer+1].h.raw = 
				p->layer[layer].h.raw +
				sizeof(struct fs_udphdr);
			break;

		case IPPROTO_ICMP:
			p->layer[layer].name = "ICMP";
			p->sport = p->layer[layer].h.icmp->type;
			p->dport = p->layer[layer].h.icmp->code;
			p->layer[layer+1].h.raw = 
				p->layer[layer].h.raw +
				sizeof(struct fs_icmphdr);

			/* TODO: check whether a packet is actually in
			* the payload or not (could be an echo etc.. */
			p->layer[layer+1].id = 0x0800;
			p->layer[layer+1].type = LTYPE_NET;
			return DEMUX_OK;
			
			break;

		case IPPROTO_IGMP:
			p->layer[layer].name = "IGMP";
			p->layer[layer+1].h.raw = 
				p->layer[layer].h.raw +
				sizeof(struct fs_igmphdr);
			break;
			
		default:
			return DEMUX_UNKNOWN;
			
	}
	
	p->layer[layer+1].type = LTYPE_APP;
	p->layer[layer+1].id = 0;
	p->layer[layer+1].name = NULL;
	return DEMUX_OK;
}
