/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <cleanup.h>
#include <loader.h>

p_cleanup_list cl_list=NULL;

int cleanup_init(void)
{
	return 1;
}

/* Add a cleanup handler */
int cleanup_add_handler(proc_cleanup handler, void *priv)
{
	p_cleanup_list n=NULL;

	if ( !handler ) return 0;

	if ( !(n=globals.malloc(sizeof(cleanup_list_t))) ){
		return 0;
	}

	n->cleanup=handler;
	n->priv=priv;
	
	n->next=cl_list;
	cl_list=n;
	return 1;
}

/* Call exit, but call all cleanup handlers first */
void cleanup(int code)
{
	p_cleanup_list tmp=NULL;
	p_cleanup_list foo=NULL;

	print_raw( CRLF "CLEANING UP" CRLF );

	print_out("num_ops: %u\n", globals.num_ops);
	
	for (tmp=cl_list; tmp; tmp=tmp->next) {
		tmp->cleanup(code, tmp->priv);
	}


	/* Special case: plugin unloading.
	* Unloading plugins makes some cleanup handlers
	* disappear. We gotta unload 'em last of all */
	loader_cleanup(code, NULL);
	
	/* Cleanup the cleanup handlers */
	for (tmp=cl_list; tmp;) {
		foo=tmp;
		tmp=tmp->next;
		globals.free(foo);
		foo=NULL;
	}
	
	print_out("Exiting with code %i" CRLF, code);

	exit(code);
}

/* Generic cleanup handler for list items with ONE pointer */
void cleanup_arg(int code, void *priv)
{
	p_config_arg foo,bar;

	for (foo=(p_config_arg)priv; foo;) {
		bar=foo;
		foo=(p_config_arg)bar->l.next;
		if (bar->arg) globals.free(bar->arg);
		globals.free(bar);
		bar=NULL;
	}
}

/* Generic cleanup handler for list items with TWO pointers */
void cleanup_kv(int code, void *priv)
{
	p_kv foo,bar;

	for (foo=(p_kv)priv; foo;) {
		bar=foo;
		foo=(p_kv)bar->l.next;
		if (bar->key) globals.free(bar->key);
		if (bar->val) globals.free(bar->val);
		globals.free(bar);
		bar=NULL;
	}
}


/* Generic cleanup handler for list items with TWO pointers */
void cleanup_c_capdev(int code, void *priv)
{
	p_c_capdev foo,bar;

	for (foo=(p_c_capdev)priv; foo;) {
		bar=foo;
		foo=(p_c_capdev)bar->l.next;
		if (bar->type) globals.free(bar->type);
		if (bar->source) globals.free(bar->source);
		if (bar->filter) globals.free(bar->filter);
		globals.free(bar);
		bar=NULL;
	}
}
