/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <s_globals.h>
#include <targets.h>
#include <cleanup.h>

p_target tgs=NULL;

void target_cleanup(int code, void *priv)
{
	p_target foo,bar;
	
	print_raw("Removing targets" CRLF);
	
	if ( tgs==NULL ) return;

	for (foo=tgs; foo; )
	{
		bar=foo;
		foo=(p_target)foo->l.next;
		globals.free(bar);
		bar=NULL;
	}
	
	return;
}

int target_init(void)
{

	cleanup_add_handler(target_cleanup, NULL);
	return 1;
}

p_target target_get(char *name)
{
	p_target p;

	for (p=tgs; p; p=(p_target)p->l.next)
	{
		if ( strcasecmp(name, p->name) == 0 ) {
			return p;
		}
	}

	return NULL;
}

/* Plugins call this to add a new target */
int target_add(char *name, proc_target tg,
	proc_target_validate v,
	proc_target_cleanup c)
{
	p_target newt=NULL;
	
	if ( !name || !tg || !v || !c ) return 0;

	if ( !(newt=globals.malloc(sizeof(target_t))) ) return 0;
	
	newt->name=name;
	newt->target=tg;
	newt->validate=v;
	newt->cleanup=c;
	
	tgs=list_append(newt, tgs);
	globals.num_targets++;

	return 1;
}
