/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define MAGIC_LBL		0xa1b2c3d4
#define MAGIC_REDHAT		0xa1b2cd34 

#include <log.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;
proc_target_add		add_target;
proc_list_append	list_append;
proc_list_remove	list_remove;

p_log_priv log_table=NULL;

target_action_e log_target(p_packet p, void *priv, p_rule r)
{
	p_log_priv lf=(p_log_priv)priv;
	struct pcap_pkthdr ph;
	int i, iplayer;
	int mylen;
	
	if ( !lf ) return TARGET_CONTINUE;
	

	/* Find the first IP header */
	for ( i=0, iplayer=-1; i<p->top_layer; i++ ) {
		if ( p->layer[i].id == 0x0800 ) {
			iplayer=i;
			break;
		}
	}

	/* If there is no IP we can't log */
	if ( iplayer == -1 ) return TARGET_DONE;
	
	/* Setup the header */
	mylen=ntohs(p->layer[iplayer].h.ip->tot_len);
	ph.ts.tv_sec=p->time;
	ph.ts.tv_usec=p->nano_time;
	ph.caplen=mylen;
	ph.len=p->len;
	
	/* Do the write */
	/* 2 syscalls per packet, bah! */
	write(lf->fd, &ph, sizeof(ph));
	write(lf->fd, p->layer[iplayer].h.raw, mylen);
	
	return TARGET_DONE;
}

p_log_priv find_log(char *fn)
{
	p_log_priv tmp;

	for (tmp=log_table; tmp; tmp=(p_log_priv)tmp->l.next) {
		if ( !strcmp(fn, tmp->fn) ) {
			return tmp;
		}
	}
	
	return NULL;
}

void log_cleanup(void *priv)
{
	p_log_priv p=(p_log_priv)priv;
	
	p->usage--;
	if ( p->usage <= 0 ) {
		list_remove(p);
		
		if ( p->fd >= 0 ) close(p->fd);
	
		my_free(p->fd);
		my_free(p);
		return;
	}
}

void *log_validate(char *token)
{
	p_log_priv x;
	struct pcap_file_header fh;
	
	if ( (x=find_log(token)) ) { /* Re-use and existing open file */
		x->usage++;
		return x;
	}else{
		/* Open a new logfile */
		if ( !(x=my_malloc(sizeof(log_priv_t))) ) {
			return NULL;
		}
		
		/* Copy the filename */
		if ( !(x->fn=my_strdup(token)) ) {
			my_free(x);
			return NULL;
		}
		
		/* Open the file */
		if ( (x->fd=open(x->fn, O_EXCL | O_WRONLY | O_TRUNC | O_CREAT, 00600)) < 0 ) {
			my_dree(x->fn);
			my_free(x);
			return NULL;
		}

		x->redhat = (sizeof(struct pcap_pkthdr) == 24) ? 1 : 0;

		/* Setup the file header */
		fh.magic = (x->redhat) ? MAGIC_REDHAT: MAGIC_LBL;
		fh.version_major=2;
		fh.version_minor=4;
		fh.thiszone=0; /* FIXME: GMT */
		fh.sigfigs=0;
		fh.linktype=DLT_RAW;
		
		x->usage=1;
		log_table=list_append(x, log_table);
		
		/* Write the header */
		write(x->fd, &fh, sizeof(fh));
		
		return x;
	}
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_target=in->add_target;
	list_append=in->list_append;
	
	PLUGIN_NAME("target/log");
	PLUGIN_DESC("Logs packets to tcpdump files");
	PLUGIN_VERS(1,0);
	
	add_target("LOG",
		log_target,
		log_validate,
		log_cleanup);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	return PLUGIN_ERR_OK;
}
